#zkmProxy external method module

from zkm.zkmRequest import ZkmRequest
from zkm.zkmCommand import ZkmCommand
from zkm.zkmSocket import ZkmSocket

from zkm import zkmUrl
from zkm import zkmFile
from zkm import zkmHtml
from zkm.zkmTrace import writeTrace, writeError, writeTracebackAsString

import urllib
import string
import time
import os

def createTempFile(tempFolder, userName, htmlPageString):
	try:
		filePathName = zkmFile.createTempFileName(tempFolder, userName)
		f = open(filePathName, "w")
	except Exception, data:
		writeError("Error creating temporary file for HTML page.")
		writeTracebackAsString(20)
		return None

	try:
		f.write(htmlPageString)
		f.close()
	except:
		writeError("Error writing temporary file for HTML page.")
		writeTracebackAsString(20)
		try:
			os.remove(filePathName)
		except:
			pass
		return None

	return filePathName

def createSavePageCommand(aZkmRequest, filePathName):
	try:
		userName = aZkmRequest.userName
		url = aZkmRequest.urlAfterOpen
		graphicsCaching = aZkmRequest.graphicsCaching

		dateAndTime = `time.time()`
		aZkmCommand = ZkmCommand(command="save", userName=userName, url=url,
								 filePathName=filePathName, graphicsCaching=graphicsCaching,
								 dateAndTime=dateAndTime)
	except Exception, data:
		writeError("Error creating 'save' command: " + `data` + "\n")
		writeTracebackAsString(20)
		return None

	return aZkmCommand

def saveAndIndexPage(command, serverName, serverPort):
	try:
		aZkmSocket = ZkmSocket(command, serverName, serverPort)
		if not aZkmSocket.sendCommandNoResponse():
			writeError("Error sending 'save' command.", command=command)
	except Exception, data:
		writeError("Error saving HTML page: " + `data` + "\n", command=command)
		writeTracebackAsString(20)

	return

def zkmProxy(self, REQUEST, AUTHENTICATED_USER):
	aZkmRequest = ZkmRequest(REQUEST, AUTHENTICATED_USER)

	#aZkmRequest.writeTrace()

	#get the contents of the url
	try:
		urlFileObj = urllib.urlopen(aZkmRequest.urlToOpen)

	except IOError, data:
		error = "IOError exception opening URL " + aZkmRequest.urlToOpen
		writeError(error)
		writeTracebackAsString(20)
		return error

	except Exception, data:
		error = "Exception opening URL " + aZkmRequest.urlToOpen
		writeError(error)
		writeTracebackAsString(20)
		return error

	list = urlFileObj.readlines()
	htmlPageString = ""
	for item in list:
		htmlPageString = htmlPageString + item

	#adjust base url for possible redirection
	aZkmRequest.urlAfterOpen = urlFileObj.geturl()
	urlFileObj.close()
	aZkmRequest.zkmBase = zkmUrl.getUrlBaseFromFullUrl(aZkmRequest.urlAfterOpen)

	tempFolder = self.aq_parent["tempFolder"]
	serverPort = self.aq_parent["serverPort"]

	#save and index HTML page read
	filePathName = createTempFile(tempFolder, aZkmRequest.userName, htmlPageString)
	if filePathName:
		command = createSavePageCommand(aZkmRequest, filePathName)
		if command:
			saveAndIndexPage(command, aZkmRequest.serverName, serverPort)

	#convert HTML for page so that user will be brought back here when any of its links are clicked
	htmlPageString = zkmHtml.proxifyHtml(aZkmRequest, htmlPageString)

	return htmlPageString

