#zkmServer stand-alone program

from zkm.zkmTrace import showError, writeTracebackAsString
from zkm.zkmZope import getOptionsDictFromZope
from zkm.zkmCommand import deserialize
from zkm.zkmSocket import cleanupSocket
from zkm.zkmDb import ZkmPageDb, ZkmPage
from zkm import zkmFile
from zkm import zkmHtml

import socket
import string
import select
import time
import sys
import os

optionsDict = {}

def addMetaDataAndSavePageToFile(commandDict):
	filePathName, url, dateAndTime, userName = (commandDict["filePathName"], commandDict["url"],
												commandDict["dateAndTime"], commandDict["userName"])
	htmlFilesFolder = optionsDict["htmlFilesFolder"]
	s = zkmFile.readFileIntoString(filePathName)
	s = zkmHtml.addMetaDataToHtmlString(s, url, dateAndTime, userName)
	filePathName = zkmFile.createHtmlFileName(htmlFilesFolder, userName)
	zkmFile.writeStringToFile(filePathName, s)
	return filePathName

def handleSaveCommand(aZkmPageDb, commandDict, graphicsCaching):
	tempFilePathName = commandDict["filePathName"]
	filePathName = addMetaDataAndSavePageToFile(commandDict)
	s = zkmHtml.removeHtml(filePathName)
	commandDict["filePathName"] = filePathName
	aZkmPage = ZkmPage(commandDict, s)
	aZkmPageDb.storeZkmPage(aZkmPage)
	if graphicsCaching:
		zkmFile.writeStringToFile(tempFilePathName, filePathName)
		baseName = os.path.basename(tempFilePathName)
		dirName = os.path.dirname(tempFilePathName)
		os.rename(tempFilePathName, os.path.join(dirName, "zkmBd_" + baseName))
	else:
		os.remove(tempFilePathName)
	return

def serve(zopeHost, zopePort):
	global optionsDict
	optionsDict = getOptionsDictFromZope(zopeHost, zopePort)
	serverPort = optionsDict["serverPort"]
	aZkmPageDb = ZkmPageDb(optionsDict["htmlFilesFolder"])
	HOST = ''
	serversock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
	serversock.bind(HOST, serverPort)
	serversock.listen(5)
	print "zkmServer is ready for requests; listening on port " + `serverPort` + "\n"

	try:
		while 1:
			time.sleep(0.1)

			input, output, exc = select.select([serversock], [], [], 0)

			for sock in input:
				try:
					handlersock, addr = sock.accept()
				except Exception, data:
					showError("zkmServer - Error accepting client socket: " + `data`)
					break

				try:
					rcvdata = handlersock.recv(1024)
				except Exception, data:
					cleanupSocket(handlersock)
					showError("zkmServer - Error receiving client data: " + `data`)
					break

				try:
					commandDict = deserialize(rcvdata)
					command = commandDict["command"]

					if command == "save":
						handlersock.close()
						userName = commandDict["userName"]
						graphicsCaching = commandDict["graphicsCaching"]
						handleSaveCommand(aZkmPageDb, commandDict, graphicsCaching)
					else:
						handlersock.close()

				except Exception, data:
					cleanupSocket(handlersock)
					showError("zkmServer - Error processing client request: " + `data`)
					writeTracebackAsString(20)
					try:
						showError(rcvdata)
					except:
						pass
					break

	except Exception, data:
		showError("zkmServer - Error: " + `data`)
		cleanupSocket(serversock)
		try:
			aZkmPageDb.close()
		except:
			showError("zkmServer - Error closing 'pageDb' database: " + `data`)

	return

if __name__ == "__main__":
	try:
		argsError = 0
		arg1, val1 = string.split(sys.argv[1], "=")
		arg2, val2 = string.split(sys.argv[2], "=")
		zopeHost = string.strip(val1)
		zopePort = int(string.strip(val2))
	except:
		argsError = 1

	if argsError or (arg1 != "zopeHost") or (arg2 != "zopePort"):
		print "Usage: python zkmServer.py zopeHost=hostname zopePort=nnnn\n"
		print "Invalid command arguments. Please see usage note above."
		sys.exit(-1)

	serve(zopeHost, zopePort)
